
/*
 * $Id: CltOpts.cc,v 1.1.1.8 1999/02/12 22:27:39 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "CltOpts.h"
#include "UniqId.h"
#include "SharedOpts.h"

#include "globals.h"

CltOpts TheCltOpts;


CltOpts::CltOpts():
	theOrgAddr(this,        "origin <host:port>", "origin host", NetAddr("localhost", 8080)),
	thePxyAddr(this,        "proxy <host:port>",  "proxy host", NetAddr("127.0.0.1", 8080)),

	theWorldType(this,      "world_type <str>",   "part of a URL", "Polygraph"),
	theWorldId(this,        "world_id <str>",     "part of a URL", Id2Str(UniqId::Create())),
	theWorldCap(this,       "world_cap <int>",    "#unique object ids", 0x7fffffff),
	theWorldUrlsFileName(this,"world_urls <file>","file with URLs"),

	genAbsUrls(this,        "abs_urls <bool>",    "generate absolute urls", true),
	theUniqStart(this,      "unique_urls <int>",  "append unique suffix (counter) to URLs"),
	theReqOrder(this,       "order <int>",        "request order"),

	thePorts(this,          "ports <port_range>", "ephemeral|range"),
	theConnPoolCap(this,    "conn_pool <int>",    "[initial] connection pool capacity"),
	adjustConnPoolSize(this,"conn_pool_dyn <bool>","dynamically adjust conn pool capacity"),
	theConnEstTimeout(0,    "conn_est_tout <time>","connection establishing timeout"),
	theLaunchWin(this,      "launch_win <time>",  "interval to start all robots"),

	theRobotCnt(this,       "robots <int>",       "number of robots"),
	//theReqCcl(0,          "req_ccl <int>",      "concurrent requests per user"),
	theReqRate(this,        "req_rate <rate>",    "user request rate"),

	doIOs(this,             "do_ios <yes|no>",    "do socket ios", true)
{
	theConnEstTimeout.argType("time");
}

bool CltOpts::validate() const {
	if (theRobotCnt < 0)
		cerr << ThePrgName << ": unspecified number of robots" << endl;
	else
	if (!TheOpts.theGoal)
		cerr << ThePrgName << ": unspecified goal" << endl;
	else
	if (theReqOrder >= 0 && TheOpts.theGoal.theXactComplCnt < 0)
		cerr << ThePrgName << ": must specify total #completions when using req order" << endl;
	else
	if (TheOpts.theXactThinkTime > 0 && theReqRate)
		cerr << ThePrgName << ": xact_think is useless used with req_rate" << endl;
	else
		return true;
	return false;
}

void CltOpts::passSeed(int seed) {
	theConnEstTimeout.rndGen()->seed(seed);
}
