
/*
 * $Id: SrvOpts.cc,v 1.1.1.3 1999/02/12 22:27:41 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "SharedOpts.h"
#include "SrvOpts.h"

#include "globals.h"

SrvOpts TheSrvOpts;


SrvOpts::SrvOpts():
	thePort(this,        "port <int>",         "listen port number"),
	theAcceptLmt(this,   "accept_lmt <int>",   "accept no more at once"),
	theTimeout(this,     "idle_tout <time>",   "exit after inactivity timeout"),
	theRepSize(this,     "rep_sz <size>",      "mean reply cont. length", new ExpDistr(new RndGen, Size::KB(13))),
	theRepCblRatio(this, "rep_cachable <ratio>","produce cachable replies")
{
	theRepSize.argType("size");
}

bool SrvOpts::validate() const {
	if (thePort < 0) {
		cerr << ThePrgName << ": no listen port specified" << endl;
		return false;
	}
	if (theRepCblRatio > 1) {
		cerr << ThePrgName << ": rep cachability ratio cannot be higher than 1.0; got: " << theRepCblRatio << endl;
		return false;
	}
	if (!theRepSize) {
		cerr << ThePrgName << ": rep size distribution must be set" << endl;
		return false;
	}

	return true;
}

void SrvOpts::passSeed(int seed) {
	theRepSize.rndGen()->seed(seed);
}
