
/*
 * $Id: CltConnMgr.h,v 1.1.1.2 1999/02/07 05:08:46 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef CLT_CONN_MGR_H
#define CLT_CONN_MGR_H

#include "Select.h"
#include "NetAddr.h"
#include "Ring.h"
#include "Farm.h"
#include "Connection.h"

class CltConnMgr: public Select::User {
	public:
		class User {
			public:
				virtual ~User() {}
				virtual void noteConnReady(Connection *conn) = 0;
		};

	public:
		CltConnMgr();
		virtual ~CltConnMgr();

		void configure(const NetAddr &anAddr, int aCcGoal, int ccLimit);
		void start();

		void order(User *u);
		void put(Connection *);

		void checkCcLevel();

		virtual void noteWriteReady(int fd);

	protected:
		void maintainCcLevel();
		Connection *open();
		Connection *open2(Connection *);
		void give(Connection *conn, User *u);

		int ccLevel() const { return thePendLevel + theReadyConns.count() + theUseLevel; }
		int prefetchLevel() const { return thePendLevel - theUsers.count(); }

		bool behindDemand() const { return theUsers.count() > thePendLevel; }
		bool behindGoal() const { return ccLevel() < theCcGoal; }

	protected:
		ObjFarm<Connection> theConns;
		Ring<Connection*> theReadyConns;
		Array<Connection*> theConnIdx;
		Ring<User*> theUsers;

		NetAddr theAddr;
		int theCcGoal;
		int thePendLevel;
		int theUseLevel;
};

#endif
