
/*
 * $Id: Progress.h,v 1.1.2.1 1999/02/05 16:52:31 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef PROGRESS_CLASS_H
#define PROGRESS_CLASS_H

#include "Clock.h"

class OLog;
class ILog;
class ostream;

// maintains information about simulation progress (or simulation goal)
class ProgrRec {
	public:
		ProgrRec();

		void store(OLog &) const;
		void load(ILog &);
		ostream &print(ostream &os) const;

	public:
		Time theDuration;
		int theXactComplCnt;
		int theErrCnt;
};

// automagically updates duration; ctor set counters to zero
class Progress: public ProgrRec, public Clock {
	public:
		Progress();

		virtual void update(Time aCurTime);

	protected:
		Time theStartTime;
};

// progress target
class Goal: public ProgrRec {
	public:
		operator void *() const; // is set?

		Goal &operator /=(double factor);
};



/* inline operators */

inline ostream &operator <<(ostream &os, const ProgrRec &r) { return r.print(os); }
inline OLog &operator <<(OLog &ol, const ProgrRec &r) { r.store(ol); return ol; }
inline ILog &operator >>(ILog &il, ProgrRec &r) { r.load(il); return il; }

inline 
bool operator >=(const ProgrRec &cur, const ProgrRec &goal) {
	return // reaching one sub-goal is enough!
		(cur.theXactComplCnt >= goal.theXactComplCnt && goal.theXactComplCnt >= 0) ||
		(cur.theErrCnt >= goal.theErrCnt && goal.theErrCnt >= 0) ||
		(cur.theDuration > goal.theDuration && goal.theDuration >= 0)
		;
}
inline 
bool operator <(const ProgrRec &cur, const ProgrRec &goal) {
	return !(cur >= goal);
}

inline 
Goal operator /(const Goal &g, double factor) { return Goal(g) /= factor; }

#endif
