
/*
 * $Id: lr.cc,v 1.1.1.2 1999/02/05 16:51:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#include "Opt.h"
#include "opts.h"

#include "StatPing.h"
#include "StatSoft.h"
#include "StatPhase.h"
#include "ILog.h"
#include "LogMeta.h"

class LROpts: public OptGrp {
	public:
		LROpts():
			theHelpOpt(this,      "help",           "list of options"),
			theOutFileName(this,  "out <file>",     "redirect console output", "-"),
			theInFileName(this,   "in <file>",      "input file", "-") 
			{}

	public:
		HelpOpt theHelpOpt;
		StrOpt theOutFileName;
		StrOpt theInFileName;
} TheOpts;

static ILog TheInLog;
static int TheTargetPhaseTag = 0;

static
ostream &loghere(ostream &os) {
	static Time startTime = TheInLog.locTime();
	return os 
		<< setw(7) << (TheInLog.locTime()-startTime).secd() 
		<< ' ' << setw(6) << TheInLog.locCount() << "| ";
}

static
bool doEntry(const LogEntryPrefix &px) {
	//cerr << here << "doing entry tag: " << px.theTag << " size: " << px.theSize << endl;

	const LogTagDescr *descr = TheInLog.tagDescr(px.theTag);
	if (!descr) {
		cerr << here << "unknown tag " << px.theTag << endl;
		return false;
	}

	switch (descr->theType) {
		case lpComment: {
			String s;
			TheInLog >> s;
			cout << loghere << s << endl;
			return true;
		}
		case lpUniqId:
		case lpStatPing:
		case lpStatSoft:
			return true; // ignore for now

		case lpStatPhase: 
			if (px.theTag == TheTargetPhaseTag) {
				StatPhaseRec ph;
				ph.load(TheInLog);
				cout << loghere << "mean resp time: " << ph.xactLifeTimeMean() << endl;
			}
			return true;

		default:
			cerr << here << "unsupported type: `" << descr->theName << "' (" << descr->theType << ")" << endl;
	}
	return false;
}

static
void configureLogs(int prec) {
	if (TheOpts.theOutFileName && strcmp(TheOpts.theOutFileName, "-"))
		cout = cerr = clog = *new ofstream(TheOpts.theOutFileName);

	if (TheOpts.theInFileName && strcmp(TheOpts.theInFileName, "-"))
		cin = *new ifstream(TheOpts.theInFileName);

	configureStream(cout, prec);
	configureStream(cerr, prec);
	configureStream(clog, prec);
}

// set some general stuff and
// propogate cmd line options to corresponding objects
static
void configure() {

	configureLogs(2);
}

int main(int argc, char *argv[]) {

	CmdLine cmd(Array<OptGrp*>() << &TheOpts);
	if (!cmd.parse(argc, argv) || !TheOpts.validate() || !TheOpts.validate())
		return -1;

	configure();

	TheInLog.stream("-", &cin);
	TheInLog.loadHeader();

	if (const LogTagDescr *descr = TheInLog.tagDescr("p-meas")) {
		TheTargetPhaseTag = descr->theTag;
	}

	while (LogEntryPrefix prefix = TheInLog.begEntry()) {
		//if !(prefix.tag != TheOpts.SearchTag && TheOpts.SearchTag > 0)
		doEntry(prefix);
		TheInLog.endEntry();
	}

	return 0;
}
