
/*
 * $Id: Select.cc,v 1.1.1.2 1999/02/05 17:15:18 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include <unistd.h>
#include <errno.h>

#include "Assert.h"
#include "Select.h"

const int Select::TheFDLimit = FD_SETSIZE; // XXX: cannot be change run-time

FD_Set::FD_Set(): theMaxFD(-1), theResCount(0) {
	Assert(Select::TheFDLimit > 0);

	FD_ZERO(&theSet);
	FD_ZERO(&theReadySet);

	theUsers.stretch(Select::TheFDLimit);
	theUsers.memset(0);
}

FD_Set::~FD_Set() {
}

void FD_Set::setFD(int fd, SelectUser *u) {
	Assert(fd >= 0 && u);
	Assert(fd < Select::TheFDLimit);

	Assert(!theUsers[fd]);
	FD_SET(fd, &theSet);
	if (fd > theMaxFD)
		theMaxFD = fd;
	theUsers[fd] = u;
	theResCount++;
}

void FD_Set::clearFD(int fd) {
	Assert(fd >= 0);

	Assert(fd <= theMaxFD);
	Assert(theUsers[fd]);
	FD_CLR(fd, &theSet);
	FD_CLR(fd, &theReadySet);
	if (fd == theMaxFD)
		while (--theMaxFD >= 0 && !FD_ISSET(theMaxFD, &theSet));
	theUsers[fd] = 0;
	theResCount--;
}


/* Select */

Select::Select(): theMaxFD(-1), theResCount(0) {
}

// returns #ready fds and -1 on error
int Select::select(Time *timeout) {

	if (timeout && *timeout < 0)
		return 0;

	// silent fix:
	// unfortunately, some selects(2) have magic limits on timeouts
	if (timeout && timeout->tv_sec > 100000000)
		timeout->tv_sec = 100000000;

	return ::select(theMaxFD+1,
		theSets[dirRead].makeReadySet(),
		theSets[dirWrite].makeReadySet(),
		0,
		timeout);
}

SelectReserv Select::setFD(int fd, IODir dir, SelectUser *u) {
	Assert(u);
	theSets[dir].setFD(fd, u);
	if (fd >= theMaxFD)
		theMaxFD = fd;
	theResCount++;
	return SelectReserv(fd, dir);
}

void Select::clearFD(int fd, IODir dir) {
	Assert(fd >= 0);
	theSets[dir].clearFD(fd);
	if (fd >= theMaxFD)
		syncMaxFD();
	theResCount--;
}

// synchronizes global maxFD with those in the sets
void Select::syncMaxFD() {
	theMaxFD = theSets[0].maxFD() > theSets[1].maxFD() ?
		theSets[0].maxFD() : theSets[1].maxFD();
}
