
/*
 * $Id: Select.h,v 1.1.1.1 1999/01/27 08:40:45 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef X_SELECT_LOOP_H
#define X_SELECT_LOOP_H

#include <sys/types.h>
#include <unistd.h>

#include "Array.h"
#include "Time.h"

// XXX: no support for exception set (we've never used one)
typedef enum { dirNone = -1, dirRead = 0, dirWrite = 1 } IODir;

// guess
class SelectUser {
	public:
		virtual ~SelectUser() {}

		virtual void noteReadReady(int /*fd*/) { Assert(0); }
		virtual void noteWriteReady(int /*fd*/) { Assert(0); }
};

// reservation receipt
class SelectReserv {
	public:
		SelectReserv(): theFD(-1), theDir(dirNone), isReady(false) {}
		SelectReserv(int aFD, IODir aDir): theFD(aFD), theDir(aDir) {}

		operator bool() const { return reserved(); }
		bool reserved() const { return theFD >= 0; }
		bool ready() const { return isReady; }
		int fd() const { return theFD; }
		IODir dir() const { return theDir; }

		void ready(bool be) { isReady = be; }
		void clear() { theFD = -1; }

	protected:
		int theFD;
		IODir theDir;
		bool isReady;
};

// wrapper arrounf fd_set
class FD_Set {
	public:
		FD_Set();
		~FD_Set();

		int maxFD() const { return theMaxFD; }
		int resCount() const { return theResCount; }
		bool active() const { return theMaxFD >= 0; }
		bool readyActive() const { return theReadyMaxFD >= 0; }
		fd_set *makeReadySet() { theReadySet = theSet; theReadyMaxFD = theMaxFD; return readyActive() ? &theReadySet : 0; }

		void setFD(int fd, SelectUser *u);
		void clearFD(int fd);

		bool isSet(int fd) const { return FD_ISSET(fd, &theSet); }
		bool isReady(int fd) const { return FD_ISSET(fd, &theReadySet); }
		SelectUser *operator [](int fd) { return theUsers[fd]; }

	protected:
		Array<SelectUser*> theUsers;
		fd_set theSet;
		fd_set theReadySet;

		int theMaxFD;
		int theReadyMaxFD;
		int theResCount;
};

// wrapper around select(2) system call
class Select {
	public:
		typedef SelectUser User;
	public:
		static const int TheFDLimit;

	public:
		Select();
		~Select() {}

		bool isIdle() const { return theMaxFD < 0; }
		int maxFD() const { return theMaxFD; }

		// returns #ready fds, -1 on error
		int select(Time *timeout = 0);

		SelectReserv setFD(int fd, IODir dir, User *p);
		void clearFD(SelectReserv &res) { clearFD(res.fd(), res.dir()); res.clear(); }
		void clearFD(int fd, IODir dir);

		FD_Set &set(IODir dir) { return theSets[dir]; }

	protected:
		void syncMaxFD();

	protected:
		FD_Set theSets[2]; // read/write
		int theMaxFD;
		int theResCount;   // #clients == #reserved fds
};

#endif
