
/*
 * $Id: String.h,v 1.1.1.6 1999/02/13 00:07:07 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef X_STRING_CLASS_H
#define X_STRING_CLASS_H

// memory storage to help with copy-on-write support
class StrBuf {
	public:
		StrBuf(): theBuf(0), theSize(0), theUseLvl(0) {}
		~StrBuf() { delete[] theBuf; }

		void set(const char *aBuf, int aSize);
		void alloc(int aSize) { theBuf = new char[theSize = aSize]; }

		StrBuf *use() { theUseLvl++; return this; }
		StrBuf *abandon() { if (!--theUseLvl) delete this; return 0; }
		
		StrBuf *clone() const;

	public:
		char *theBuf;
		int theSize;
		mutable int theUseLvl;
};

// a simple string class; to be improved later if needed
class String {
	public:
		String(const char *s = 0): theBuf(0) { set(s); }
		String(const String &s): theBuf(0) { set(s); }
		~String() { set(0); }

		String &operator =(const String &s) { set(s); return *this; }
		String &operator =(const char *s) { set(s); return *this; }

		char *buf() { aboutToMod(); return theBuf ? theBuf->theBuf : 0; }
		const char *str() const { return theBuf ? theBuf->theBuf : 0; }
		operator const char *() const { return str(); }
		int size() const { return theBuf ? theBuf->theSize : 0; }
		int len() const { return size() - 1; }

		char operator [](int idx) const { return theBuf->theBuf[idx]; }
		//char &operator [](int idx) { return theBuf[idx]; }
		// substring: [min, maxp1)
		String operator ()(int min, int maxp1) const;

		// case sensitive
		bool operator ==(const String &s) const;
		bool operator !=(const String &s) const { return !(*this == s); }
		bool operator ==(const char *s) const { return *this == (String)s; }
		bool operator !=(const char *s) const { return *this != (String)s; }

		void alloc(int size);

	protected:
		String(StrBuf &aBuf): theBuf(&aBuf) { theBuf->use(); }

		void set(const char *s);
		void set(const String &s);
		void aboutToMod();

	protected:
		StrBuf *theBuf;
};

extern ostream &operator <<(ostream &os, const String &s);

#endif
